#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "QTime"
#include "QTimer"
#include "QSound"
#include "QFile"
#include <stdlib.h>
#include <math.h>
#include <QtCore/qendian.h>
#include <QtMultimedia/QAudioDeviceInfo>
#include <QtMultimedia/QAudioInput>
#include <QtMultimedia/QAudioFormat>
#include <QDebug>


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    m_device(QAudioDeviceInfo::defaultInputDevice())
{
    prevLevel = 0;
    maxLevel = 2;

    m_pullMode = true;

    m_format.setFrequency(8000);
    m_format.setChannels(1);
    m_format.setSampleSize(16);
    m_format.setSampleType(QAudioFormat::SignedInt);
    m_format.setByteOrder(QAudioFormat::LittleEndian);
    m_format.setCodec("audio/pcm");

    QAudioDeviceInfo info(QAudioDeviceInfo::defaultInputDevice());
    if (!info.isFormatSupported(m_format)) {
        qWarning() << "Default format not supported - trying to use nearest";
        m_format = info.nearestFormat(m_format);
    }

    m_audioInfo  = new AudioInfo(m_format, this);
    connect(m_audioInfo, SIGNAL(update()), this, SLOT(updateLevel()));


    m_audioInput = new QAudioInput(m_device, m_format, this);
    qWarning() << "INFO";
    m_audioInfo->start();
    qWarning() << "input";
    m_audioInput->start(m_audioInfo);

    QTimer::singleShot(3000,this,SLOT(setLevelAverage()));

    //QSound soundObject = new QSound("sounds/warning.wav");
    ui->setupUi(this);
    startClock();
    time_from_start = 0;
    connect(ui->bsetAlarm, SIGNAL(clicked()), this,SLOT(setAlarm()));

    //ui->label->disconnect(connect()
    //connect(currentLevel, SIGNAL())
}


MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::playAlarm()
{

    if(alarm_state == 1) {
        ui->bsetAlarm->setText("Stop Alarm");
        soundObject->setLoops(1);
        soundObject->play();
        alarm_state = 2;
        QTimer::singleShot(200,this,SLOT(playAlarm()));
    }else if(alarm_state == 2) {

        if(soundObject->isFinished())
        {

            //soundObject->play();
            time_from_start += 2;
            alarm_state = 1;
            if(time_from_start < 60)
                QTimer::singleShot(2000,this,SLOT(playAlarm()));
            else
            {
                time_from_start = 0;
                QTimer::singleShot(60000,this,SLOT(playAlarm()));
            }
        }
        else
        {
            time_from_start += 0.1;
            QTimer::singleShot(200,this,SLOT(playAlarm()));
        }
    }/*else if(alarm_state == 3)
    {
        soundObject->play();
        time_from_start += 2;
        if(time_from_start < 60)
            QTimer::singleShot(2000,this,SLOT(playAlarm()));
        else
        {
            alarm_state = 1;
            //time_from_start = 0;
            QTimer::singleShot(100,this,SLOT(playAlarm()));
        }
    }*/
}

void MainWindow::timeControl()
{
    ui->timeLabel->setText(QTime::currentTime().toString());
    QTimer::singleShot(1000,this,SLOT(timeControl()));
}

void MainWindow::startClock()
{
    ui->timeLabel->setText(QTime::currentTime().toString());
    if (alarm_state == 1)
        if(QTime::currentTime().toString() == alarmTime)
        {
            playAlarm();
            ui->bsetAlarm->setText("Stop Alarm");
        }

    QTimer::singleShot(1000,this,SLOT(startClock()));
}

void MainWindow::setAlarm()
{
  if (ui->bsetAlarm->text() == "Set Alarm")
  {
      initializeAlarm();
      ui->bsetAlarm->setText("Unset Alarm");
      alarm_state = 1;
      alarmTime = ui->selectedTime->time().toString();
      ui->testLabel->setText(alarmTime);
      //ui->testLabel->setText(ui->selectedTime->time().toString());
      //ui->selectedTime->disconnect();
  }else  if (ui->bsetAlarm->text() == "Unset Alarm")
  {
      ui->bsetAlarm->setText("Set Alarm");
      alarm_state = 0;
      ui->testLabel->setText("");
      //ui->selectedTime->connect();
  }else  if (ui->bsetAlarm->text() == "Stop Alarm")
  {
      soundObject->stop();
      ui->bsetAlarm->setText("Set Alarm");
      alarm_state = 0;
      ui->testLabel->setText("");
  }

}

void MainWindow::initializeAlarm()
{
    soundObject = new QSound("sounds/warning.wav");
    //soundObject->fileName("sounds/warning.wav");
    if(!soundObject->isAvailable())
        ui->testLabel->setText("Error");
    //if(QFile::exists("sounds/warning.wav"))
        //ui->testLabel->setText(soundObject->fileName());
}

void MainWindow::updateLevel(){
    qreal currentLevel = m_audioInfo->level();

    if(prevLevel < currentLevel)
        prevLevel = currentLevel;



    if(currentLevel < maxLevel){
        ui->label_trigger->setText("max:"+QString::number(float(maxLevel)));
        ui->label_current->setText("Curent: "+QString::number(float(currentLevel)));
    }else
        ui->label_current->setText("Reached "+QString::number(float(currentLevel)));
    ui->soundBar->setValue(int(float(currentLevel)*100));

}



void MainWindow::setLevelAverage(){
    if(prevLevel )
    maxLevel = prevLevel;
}

